unit MemTables;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  BDE, DB, DBTables;

type
  TMemoryTable = class(TDBDataSet)
  private
    FCloneDataSet: TDBDataSet;
    FTableName: TFileName;
    procedure SetCloneDataSet(Value: TDBDataSet);
    procedure SetTableName(const Value: TFileName);
  protected
    function CreateHandle: HDBICur; override;
    procedure EncodeFieldDesc(var FieldDesc: FLDDesc;
      const Name: string; DataType: TFieldType; Size, Precision: Word);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function IsFieldDefsStored: Boolean;
  published
    property CloneDataset: TDBDataSet read FCloneDataSet write SetCloneDataSet;
    property FieldDefs stored IsFieldDefsStored;
    property TableName: TFileName read FTableName write SetTableName;
  end;

procedure Register;

implementation

uses DBCommon;

procedure Register;
begin
  RegisterComponents('Samples', [TMemoryTable]);
end;

// TMemoryTable

procedure TMemoryTable.Notification;
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = FCloneDataSet then CloneDataSet := nil
end;

function TMemoryTable.CreateHandle: HDBICur;
var
  OldActive: Boolean;
  DSProps: CurProps;
  FieldDesc: TFieldDescList;
  I, FieldCount: Word;
begin
  FieldCount := FieldDefs.Count;
  if FCloneDataSet <> nil then
  begin
    OldActive := FCloneDataSet.Active;
    try
      FCloneDataSet.Open;
      Check(DbiGetCursorProps(FCloneDataSet.Handle, DSProps));
      FieldCount := DSProps.iFields;
      SetLength(FieldDesc, FieldCount);
      Check(DbiGetFieldDescs(FCloneDataSet.Handle, pFLDDesc(FieldDesc)));
    finally
      FCloneDataSet.Active := OldActive;
    end;
  end
  else if FieldCount > 0 then
  begin
    SetLength(FieldDesc, FieldCount);
    for I := 0 to FieldCount - 1 do
      with FieldDefs[I] do
        EncodeFieldDesc(FieldDesc[I], Name, DataType, Size, Precision);
  end
  else
    DatabaseError('Dataset required');
  Check(DbiCreateInMemTable(DBHandle, PChar(FTableName), FieldCount,
    pFLDDesc(FieldDesc), Result));
end;

procedure TMemoryTable.EncodeFieldDesc(var FieldDesc: FLDDesc;
  const Name: string; DataType: TFieldType; Size, Precision: Word);
begin
  with FieldDesc do
  begin
    StrPLCopy(szName, Name, SizeOf(szName) - 1);
    iFldType := FldTypeMap[DataType];
    iSubType := FldSubTypeMap[DataType];
    case DataType of
      ftString, ftFixedChar, ftBytes, ftVarBytes, ftBlob..ftTypedBinary:
        iUnits1 := Size;
      ftBCD:
        begin
          { Default precision is 32, Size = Scale }
          if (Precision > 0) and (Precision <= 32) then
            iUnits1 := Precision else
            iUnits1 := 32;
          iUnits2 := Size;  {Scale}
        end;
    end;
  end;
end;

procedure TMemoryTable.SetCloneDataSet(Value: TDBDataSet);
begin
  if FCloneDataSet <> Value then
  begin
    CheckInactive;
    FCloneDataSet := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TMemoryTable.SetTableName(const Value: TFileName);
begin
  CheckInactive;
  FTableName := Value;
end;

function TMemoryTable.IsFieldDefsStored: Boolean;
begin
  Result := FieldDefs.Count > 0;
end;

end.
